package com.fs.app.controller.course;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import cn.hutool.core.date.DateTime;
import com.alibaba.fastjson.JSON;
import com.fs.app.controller.AppBaseController;
import com.fs.common.core.domain.R;
import com.fs.common.core.redis.RedisCache;
import com.fs.common.param.LoginParam;
import com.fs.core.config.WxMaConfiguration;
import com.fs.course.config.CourseMaConfig;
import com.fs.app.utils.JwtUtils;
import com.fs.his.domain.FsUser;
import com.fs.his.domain.FsUserWx;
import com.fs.his.mapper.FsUserLoginLogMapper;
import com.fs.his.service.IFsUserService;
import com.fs.his.service.IFsUserWxService;
import com.fs.his.utils.ConfigUtil;
import com.fs.system.domain.SysConfig;
import com.fs.system.mapper.SysConfigMapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.Synchronized;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Supplier;

/**
 * 微信小程序用户接口
 *
 */
@Api("企微看课登录接口")
@RestController
@RequestMapping(value="/app/wx")
public class CourseQwLoginController extends AppBaseController {
    private final Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private ConfigUtil configUtil;
    @Autowired
    private IFsUserService userService;
    @Autowired
    FsUserLoginLogMapper fsUserLoginLogMapper;
    @Autowired
    private SysConfigMapper sysConfigMapper;
    @Autowired
    JwtUtils jwtUtils;
    @Autowired
    RedisCache redisCache;


    @ApiOperation("小程序看课登录")
    @PostMapping("/courseLogin")
    @Transactional
    public R courseLogin(@RequestBody LoginParam param) {
        SysConfig sysConfig3 = sysConfigMapper.selectConfigByConfigKey("courseMa.config");
        List<CourseMaConfig> courseMaConfigs = JSON.parseArray(sysConfig3.getConfigValue(), CourseMaConfig.class);
        if (courseMaConfigs.isEmpty()){
            return R.error("小程序配置为空");
        }
        // 根据appId匹配配置
        CourseMaConfig matchedConfig = courseMaConfigs.stream()
                .filter(config -> param.getAppId().equals(config.getAppid()))
                .findFirst()
                .orElse(null);

        if (matchedConfig == null) {
            return R.error("未找到匹配的小程序配置");
        }
        return handleCourseLogin(param,
                () -> WxMaConfiguration.getMaService(matchedConfig.getAppid()),
                matchedConfig.getName());
    }

    /**
     * 公共登录处理方法
     * @param param 登录参数
     * @param wxServiceSupplier 微信服务提供函数（差异化点1：不同方式获取WxMaService）
     * @param logName 日志名称（差异化点2：不同场景标识）
     */
    private R handleCourseLogin(LoginParam param, Supplier<WxMaService> wxServiceSupplier, String logName) {
        if (StringUtils.isBlank(param.getCode())) {
            return R.error("code不存在");
        }

        try {
            // 通过函数式接口获取不同的微信服务实例
            final WxMaService wxService = wxServiceSupplier.get();
            WxMaJscode2SessionResult session = wxService.getUserService().getSessionInfo(param.getCode());
            this.logger.info("获取{} Session：{}", logName, session);

            FsUser user = userService.selectFsUserByUnionid(session.getUnionid());
            boolean isNewUser = false;

            // 用户存在时的更新逻辑
            if(user != null){
                FsUser userMap = new FsUser();
                userMap.setUserId(user.getUserId());
                userMap.setCourseMaOpenId(session.getOpenid());
                userMap.setUpdateTime(new DateTime());
                userService.updateFsUser(userMap);
            }
            // 用户不存在时的创建逻辑
            else {
                user = new FsUser();
                user.setNickName("微信用户");
                user.setAvatar("https://hos-1309931967.cos.ap-chongqing.myqcloud.com/fs/20230725/a848605591384ec29d49773dd58d9345.jpg");
                user.setStatus(1);
                user.setCourseMaOpenId(session.getOpenid());
                user.setUnionId(session.getUnionid());
                user.setCreateTime(new Date());
                userService.insertFsUser(user);
                isNewUser = true;
            }

            // 4. 处理用户与小程序的绑定
            handleFsUserWx(user, param,session);

            // 生成Token
            String token = jwtUtils.generateToken(user.getUserId());
            Map<String,Object> map = new HashMap<>();
            map.put("token", token);
            map.put("user", user);
            map.put("isNew", isNewUser);

            logger.info("zyp \n 【点播{}登录】：{}", logName, user.getUserId());
            return R.ok(map);
        } catch (WxErrorException e){
            logger.error("{}授权失败：{}", logName, e.getMessage(), e);
            return R.error("授权失败," + e.getMessage());
        }
    }


    @Autowired
    private IFsUserWxService fsUserWxService;



    /**
     * 处理用户与小程序的绑定
     */
    private void handleFsUserWx(FsUser user, LoginParam param,WxMaJscode2SessionResult session) {
        if (user == null) return;
        // 尝试更新
        boolean updated = fsUserWxService.lambdaUpdate()
                .eq(FsUserWx::getFsUserId, user.getUserId())
                .eq(FsUserWx::getAppId, param.getAppId())
                .eq(FsUserWx::getOpenId, session.getOpenid())
                .set(FsUserWx::getUnionId, session.getUnionid() == null ? "" : session.getUnionid())
                .set(FsUserWx::getUpdateTime, new Date())
                .update();

        // 如果更新失败（记录不存在），则插入
        if (!updated) {
            FsUserWx fsUserWx = new FsUserWx();
            fsUserWx.setType(1);
            fsUserWx.setFsUserId(user.getUserId());
            fsUserWx.setAppId(param.getAppId());
            fsUserWx.setOpenId(session.getOpenid());
            fsUserWx.setUnionId(session.getUnionid() == null ? "" : session.getUnionid());
            fsUserWx.setCreateTime(new Date());
            fsUserWx.setUpdateTime(new Date());
            fsUserWxService.save(fsUserWx);
        }
        logger.info("zyp \n 【更新或插入用户与小程序{}的绑定关系】：{}", param.getAppId(), user.getUserId());

    }


    public static String genCode() {
        String year = new SimpleDateFormat("yy").format(new Date());
        String day = String.format("%tj", new Date());
        double random = Math.random() * 10000000;
        while (random < 1000000) {
            random = Math.random() * 10000000;
        }
        int intRandom = Double.valueOf(random).intValue();
        String verifyCode = year + day + intRandom;
        return verifyCode;
    }


    @GetMapping("/loginTest")
    @Synchronized
    public R loginTest(String phone) {
        //如果开启了UnionId
        FsUser user=userService.selectFsUserByMpOpenId(phone);
        String token = jwtUtils.generateToken(user.getUserId());
        return R.ok("登录成功").put("token",token).put("user", user);
    }

}
